%Simulates a 1-dimensional Potts model with Glauber dynamics.
%Nearest-neighbor; update each site with rate one.

q=4;         %number of types
beta=3.3;    %strength of the interaction
L=300;       %number of sites
Tot=300;     %total time
Steps=200;   %number of times when the state is recorded

X=ceil(q*rand(1,L));  %iid uniform initial state

begin=clock;

Evol=zeros(Steps,L);          %this is where the picture will be stored
Times=linspace(0,Tot,Steps);  %list of times when the state is recorded
tt=1/L;                  %duration of one step of the simulation.
time=-tt;                %real time
step=1;
while step<=Steps        %while there is still a time to be recorded
 time=time+tt;           %increase real time
 if time>=Times(step)
  Evol(step,:)=X;        %record the state
  step=step+1;           %increase counter of next state to be recorded
 end
 site=ceil(L*rand);      %choose random site
 ll=mod(site-2,L)+1;     %left neighbor with periodic b.c.
 rr=mod(site,L)+1;       %right neighbor with periodic b.c.
 N=zeros(1,q);           %list of frequencies of types among neighbors
 N(X(ll))=N(X(ll))+1;
 N(X(rr))=N(X(rr))+1;
 prob=zeros(1,q);        %probability to update to a given type
 for type=1:q
  prob(type)=exp(beta*N(type));
 end
 prob=prob/sum(prob);    %normalization
 X(site)=choose(prob);   %new type with law prob
end

elaps=etime(clock,begin);
elaps=elaps/(60*60);
hours=floor(elaps)
elaps=60*(elaps-hours);
minutes=floor(elaps)
seconds=60*(elaps-minutes)

Q=flipud(Evol);          %time runs upwards
Pic=makepic(Q,q);
imshow(Pic)
