%Simulates a 1-dimensional nearest-neighbor contact process.

lambda=2;    %infection rate
L=300;       %number of sites
Tot=100;     %total time
Steps=200;   %number of times when the state is recorded

dens=0.02;             %product measure initial state with intensity dens
X=(rand(1,L)<dens);

%mid=ceil(L/2);        %initial state with a single infected site
%X=zeros(1,L);
%X(mid)=1;

begin=clock;

Evol=zeros(Steps,L);          %this is where the picture will be stored
Times=linspace(0,Tot,Steps);  %list of times when the state is recorded
tt=1/(L*(1+2*lambda));   %duration of one step of the simulation.
p=1/(1+2*lambda);        %probability of recovery
time=-tt;                %real time
step=1;
while step<=Steps        %while there is still a time to be recorded
 time=time+tt;           %increase real time
 if time>=Times(step)
  Evol(step,:)=X;        %record the state
  step=step+1;           %increase counter of next state to be recorded
 end
 site=ceil(L*rand);      %choose random site
 if rand<p
  X(site)=0;             %recovery
 else
  if X(site)==1
   if rand<0.5
    nn=mod(site-2,L)+1; %left neighbor with periodic b.c.
   else
    nn=mod(site,L)+1;   %right neighbor with periodic b.c.
   end
   X(nn)=1;              %infect neighbor
  end
 end 
end

elaps=etime(clock,begin);
elaps=elaps/(60*60);
hours=floor(elaps)
elaps=60*(elaps-hours);
minutes=floor(elaps)
seconds=60*(elaps-minutes)

Q=flipud(2*ones(Steps,L)-Evol);  %time runs upwards, convert to states 1 and 2.
q=2;                           %number of types
Pic=makepic(Q,q);
imshow(Pic)
