%Simulates a 1-dimensional process of coalescing random walks
%with cooperative branching. Particles jump with rate 1 and branch with rate
%lamda in a randomly chosen direction. Branching can only happen if the
%site in the opposite direction is occupied.

beta=2.3;      %cooperative branching rate
L=300;       %number of sites
Tot=300;     %total time
Steps=200;   %number of times when the state is recorded

dens=0.1;              %product measure initial state with intensity dens
X=(rand(1,L)<dens);

%mid=ceil(L/2);        %initial state with two infected sites
%X=zeros(1,L);
%X(mid)=1;
%X(mid+1)=1;

begin=clock;

Evol=zeros(Steps,L);          %this is where the picture will be stored
Times=linspace(0,Tot,Steps);  %list of times when the state is recorded
tt=1/(L*(1+beta));       %duration of one step of the simulation.
p=1/(1+beta);            %probability of coalescing random walk step
time=-tt;                %real time
step=1;
while step<=Steps        %while there is still a time to be recorded
 time=time+tt;           %increase real time
 if time>=Times(step)
  Evol(step,:)=X;        %record the state
  step=step+1;           %increase counter of next state to be recorded
 end
 site=ceil(L*rand);      %choose random site
 if X(site)==1           %only do something if site is occupied
  if rand<0.5            %jump or branch to the left
   ll=mod(site-2,L)+1;   %left neighbor with periodic b.c.
   if rand<p             %jump
    X(site)=0;
    X(ll)=1;
   else                  %branch to left if right neighbor occupied
    rr=mod(site,L)+1;    %right neighbor with periodic b.c.
    if X(rr)==1
     X(ll)=1;
    end
   end
  else                   %jump or branch to the right
   rr=mod(site,L)+1;     %right neighbor with periodic b.c.
   if rand<p             %jump
    X(site)=0;
    X(rr)=1;
   else                  %branch to right if leftt neighbor occupied
    ll=mod(site-2,L)+1;  %left neighbor with periodic b.c.
    if X(ll)==1
     X(rr)=1;
    end
   end
  end
 end 
end

elaps=etime(clock,begin);
elaps=elaps/(60*60);
hours=floor(elaps)
elaps=60*(elaps-hours);
minutes=floor(elaps)
seconds=60*(elaps-minutes)

Q=flipud(2*ones(Steps,L)-Evol);  %time runs upwards, convert to states 1 and 2.
q=2;                             %number of types
Pic=makepic(Q,q);
imshow(Pic)
