%If q is an integer greater than one, then this function translates
%a matrix Q with values in {1,...,q} into a color picture.
%If Q is three-dimensional, then three sides of the rectangle are shown.
%If q is a vector of length two, then Q(:,:,1) or Q(:,:,:,1) should
%have values in {1,...,q(1)} and is translated into colors as before,
%while Q(:,:,2) or Q(:,:,:,2) should have values in {0,...,q(2)} and
%is translated into dark/light.
function retval=makepic(Q,q)
 dim=ndims(Q);
 if length(q)==2        %if q is a vector of length two, then
  qq=q(2);              %rename q(2) into qq
  q=q(1);               %rename q(1) into q
  dim=dim-1;            %rename Q(:,:,1) or Q(:,:,:,1) into Q
  if dim==2             %and rename Q(:,:,2) or Q(:,:,:,2) into QQ
   QQ=squeeze(Q(:,:,2));
   Q=squeeze(Q(:,:,1));
  else
   QQ=squeeze(Q(:,:,:,2));
   Q=squeeze(Q(:,:,:,1));
  end
  double=1;             %record that there is a QQ
 else
  double=0;
 end
 if dim==3              %show three sides of a rectangle
  up=squeeze(Q(:,:,1));
  right=squeeze(Q(:,size(Q)(2),:));
  down=squeeze(Q(size(Q)(1),:,:))';
  Q=[up right; down zeros(size(Q)(3),size(Q)(3))];
  if double==1          %do the same with QQ if needed
   up=squeeze(QQ(:,:,1));
   right=squeeze(QQ(:,size(QQ)(2),:));
   down=squeeze(QQ(size(QQ)(1),:,:))';
   QQ=[up right; down qq*ones(size(QQ)(3),size(QQ)(3))]; %corner highest value
  end
 end
 if min(q==2,double==0)  %black and white picture when q=2 and there is no QQ
  R=(Q==2);
  G=(Q==2);
  B=(Q==2);
 end
 if min(q==2,double==1)  %blue and red picture when q=2 and there is a QQ
  R=(Q==1);
  G=zeros(size(Q)(1),size(Q)(2));
  B=(Q==2);
 end
 if max(q==3,q==4)       %when q=3 or 4 use red,yellow,blue, and green  
  R=(Q==1)+(Q==3);
  G=(Q>2);
  B=(Q==2);
 end
 if q>4                                %for q>4 use colors in the plane R+G+B=3
  R=ceil(128*(1+sin(2*pi*(Q/q))))/256; %you turn around the point R=G=B=1
  G=ceil(128*(1+sin(2*pi*(Q/q+1/3))))/256;
  B=ceil(128*(1+sin(2*pi*(Q/q+2/3))))/256;
 end
 if dim==3                             %make corner white
  R(size(up)(1)+1:size(Q)(1),size(up)(1)+1:size(Q)(1))=1;
  G(size(up)(1)+1:size(Q)(1),size(up)(1)+1:size(Q)(1))=1;
  B(size(up)(1)+1:size(Q)(1),size(up)(1)+1:size(Q)(1))=1;
 end
 if double==1 %if there is a QQ, then its values are
  P=QQ/qq;    %translated into brightness; zero is black
  R=R.*P;     %when dim=3, then corner has P=1 so remains white
  G=G.*P;
  B=B.*P;
 end
 retval=zeros(size(Q)(1),size(Q)(2),3);
 retval(:,:,1)=R;
 retval(:,:,2)=G;
 retval(:,:,3)=B;
endfunction
