%Simulates a rebellious voter model.
%Each site becomes active with rate 1 and then chooses a random direction.
%With probabily alpha, the site flips if it differs from its neighbor.
%With probabily 1-alpha, the site flips if its neighbor differs from the
%next nearest neighbor.

alpha=0.5;   %model parameter
L=300;       %number of sites
Tot=300;     %total time
Steps=200;   %number of times when the state is recorded

X=(rand(1,L)<0.5);    %product measure initial state

%mid=ceil(L/2);        %initial state with interface
%X=zeros(1,L);
%X(1:mid)=ones(1,mid);

begin=clock;

Evol=zeros(Steps,L);          %this is where the picture will be stored
Times=linspace(0,Tot,Steps);  %list of times when the state is recorded
tt=1/L;                  %duration of one step of the simulation.
time=-tt;                %real time
step=1;
while step<=Steps        %while there is still a time to be recorded
 time=time+tt;           %increase real time
 if time>=Times(step)
  Evol(step,:)=X;        %record the state
  step=step+1;           %increase counter of next state to be recorded
 end
 site=ceil(L*rand);      %choose random site
 if rand<0.5             %look to the left
  ll=mod(site-2,L)+1;    %left neighbor with periodic b.c.
  if rand<alpha          %voter move
   X(site)=X(ll);
  else                   %rebellious move
   lll=mod(site-3,L)+1;  %left next neighbor with periodic b.c.
   X(site)=mod(X(site)+X(ll)+X(lll),2);
  end
 else                    %look to the right
  rr=mod(site,L)+1;      %rightt neighbor with periodic b.c.
  if rand<alpha          %voter move
   X(site)=X(rr);
  else                   %rebellious move
   rrr=mod(site+1,L)+1;  %rightt next neighbor with periodic b.c.
   X(site)=mod(X(site)+X(rr)+X(rrr),2);
  end
 end
end

elaps=etime(clock,begin);
elaps=elaps/(60*60);
hours=floor(elaps)
elaps=60*(elaps-hours);
minutes=floor(elaps)
seconds=60*(elaps-minutes)

Q=flipud(2*ones(Steps,L)-Evol);  %time runs upwards, convert to states 1 and 2.
q=2;                             %number of types
Pic=makepic(Q,q);
imshow(Pic)
