%Simulates a general 3-dimensional spin flip model.
%The program depends on the function fliprates(x,mm,q)
%that takes as input the state x of the site, which takes values in {1,..,q}
%and the number of neighbors in each of the q states, which is a vector
%nn of length q with values in {0,1,2,...}, and that produces as output
%a vector rates of length q that gives the rate at which x flips to the
%new value q.

q=3;                       %number of states
L=10; 		           %size of the box
Times=linspace(0,10,101);  %times when the state is recorded

%X=ones(L,L,L);             %constant initial state
X=ceil(q*rand(L,L,L));      %product initial law

begin=clock;

Steps=length(Times);
magnet=zeros(Steps,q);     %record total magnetization
M=zeros(L,L,L,q);          %local magnetization
Rs=zeros(L,L,L,q);         %local flip rates to each state
for k=1:L
 for l=1:L
  for m=1:L
   nn=neigh([k l m],L);    %coordinates of the 6 neighboring sites
   for n=1:6               %add contributions of neighors to magnetization
    nu=nn(n,:);
    y=X(nu(1),nu(2),nu(3));
    M(k,l,m,y)=M(k,l,m,y)+1;
   end
   Rs(k,l,m,:)=fliprates(X(k,l,m),M(k,l,m,:),q); %local flip rates to each state
  end
 end
end
R=sum(Rs,4);               %total local flip rate
RR=sum(sum(sum(sum(R))));  %total flip rate
tt=0;                      %duration of current step of the simulation.
time=0;                    %real time
step=1;
while step<=Steps          %while there is still a time frame to be recorded
 time=time+tt;             %increase real time
 if time>=Times(step)
  Film{1,step}=X;          %record the state
  for y=1:q
   magnet(step,y)=sum(sum(sum(X==y)))/(L*L*L);  %record total magnetization
  end
  step=step+1;             %increase counter of next state to be recorded
 end
 ss=choose3(R/RR);         %choose proportional to total local flip rate
 Rl=squeeze(Rs(ss(1),ss(2),ss(3),:));  %rates of jump to each state
 Xold=X(ss(1),ss(2),ss(3));
 Xnew=choose(Rl/sum(Rl));              %new value chosen proportional to rates
 X(ss(1),ss(2),ss(3))=Xnew;
 Rs(ss(1),ss(2),ss(3),:)=fliprates(X(ss(1),ss(2),ss(3)),M(ss(1),ss(2),ss(3),:),q);
 nn=neigh(ss,L);          %coordinates of the 6 neighboring sites
 for n=1:6                %change the magnetization and flip rates
  nu=nn(n,:);             %of neighbors
  M(nu(1),nu(2),nu(3),Xold)=M(nu(1),nu(2),nu(3),Xold)-1;
  M(nu(1),nu(2),nu(3),Xnew)=M(nu(1),nu(2),nu(3),Xnew)+1;
  Rs(nu(1),nu(2),nu(3),:)=fliprates(X(nu(1),nu(2),nu(3)),M(nu(1),nu(2),nu(3),:),q);
 end
 R=sum(Rs,4);               %total local flip rate
 RR=sum(sum(sum(sum(R))));  %total flip rate
 if RR>0
  tt=1/RR;                  %size of the time step is inverse of total rate
 else
  while step<=Steps         %if you are in a trap, stay constant
   Film{1,step}=X;          %record the state
   for y=1:q
    magnet(step,y)=sum(sum(sum(X==y)))/(L*L);  %record total magnetization
   end
   step=step+1;             %increase counter of next state to be recorded
  end
 end
end

elaps=etime(clock,begin);
elaps=elaps/(60*60);
hours=floor(elaps)
elaps=60*(elaps-hours);
minutes=floor(elaps)
seconds=60*(elaps-minutes)

%figure(1)
step=1;
Pic=makepic(Film{1,step},q); %Show the initial state
imshow(Pic)

%figure(2)
%hold off
%plot(Times,magnet(:,1),'r')
%hold on
%plot(Times,magnet(:,2),'b')
%plot(Times,magnet(:,3),'y')
%
%figure(1)
